from django.contrib.auth.models import User, Group
from django.utils.translation import ugettext_lazy as _
from django import forms
from importloaders import keepass
from keepassdb.exc import AuthenticationError, InvalidDatabase
from cred.models import CredAudit


class AuditFilterForm(forms.Form):
    hide = forms.MultipleChoiceField(
        choices=CredAudit.CREDAUDITCHOICES,
        widget=forms.SelectMultiple(attrs={'class': 'selectize-multiple'}),
        initial=[],
    )


class UserForm(forms.ModelForm):
    # We want two password input boxes
    newpass = forms.CharField(
        widget=forms.PasswordInput(attrs={'class': 'btn-password-visibility'}),
        required=False,
        max_length=32,
        min_length=8
    )
    confirmpass = forms.CharField(
        widget=forms.PasswordInput,
        required=False,
        max_length=32,
        min_length=8
    )

    # Define our model
    class Meta:
        model = User
        fields = ('username', 'email', 'is_active', 'is_staff', 'groups')
        widgets = {
            'groups': forms.SelectMultiple(attrs={'class': 'rattic-group-selector'}),
        }

    def clean(self):
        # Check the passwords given match
        cleaned_data = super(UserForm, self).clean()
        newpass = cleaned_data.get("newpass")
        confirmpass = cleaned_data.get("confirmpass")

        if newpass != confirmpass:
            msg = _('Passwords do not match')
            self._errors['confirmpass'] = self.error_class([msg])
            del cleaned_data['newpass']
            del cleaned_data['confirmpass']

        return cleaned_data


class GroupForm(forms.ModelForm):
    class Meta:
        model = Group
        fields = ('name',)


class KeepassImportForm(forms.Form):
    file = forms.FileField()
    password = forms.CharField(
        max_length=50,
        widget=forms.PasswordInput(attrs={'class': 'btn-password-visibility'})
    )
    group = forms.ModelChoiceField(
        queryset=Group.objects.all(),
        widget=forms.Select(attrs={'class': 'rattic-group-selector'}),
    )

    def __init__(self, requser, *args, **kwargs):
        super(KeepassImportForm, self).__init__(*args, **kwargs)
        self.fields['group'].queryset = Group.objects.filter(user=requser)

    def clean(self):
        cleaned_data = super(KeepassImportForm, self).clean()

        try:
            db = keepass(cleaned_data['file'], cleaned_data['password'])
            cleaned_data['db'] = db
        except AuthenticationError:
            msg = _('Could not read keepass file, the password you gave may not be correct.')
            self._errors['file'] = self.error_class([msg])
            del cleaned_data['file']
            del cleaned_data['password']
        except InvalidDatabase:
            msg = _('That file does not appear to be a valid KeePass file.')
            self._errors['file'] = self.error_class([msg])
            del cleaned_data['file']
            del cleaned_data['password']

        return cleaned_data
